==> json-joy-2.7.0.tgz: package/esm/json-crdt/api/DocumentApi.js <==
import { StringType } from "../types/rga-string/StringType";
import { PatchBuilder } from "../../json-crdt-patch/PatchBuilder";
import { Patch } from "../../json-crdt-patch/Patch";
import { NoopOperation } from "../../json-crdt-patch/operations/NoopOperation";
import { LogicalTimestamp } from "../../json-crdt-patch/clock";
import { StringApi } from "./StringApi";
import { NumberType } from "../types/lww-number/NumberType";
import { ArrayType } from "../types/rga-array/ArrayType";
import { ObjectType } from "../types/lww-object/ObjectType";
import { UNDEFINED_ID } from "../../json-crdt-patch/constants";
export class DocumentApi {
    constructor(doc) {
        this.doc = doc;
        this.patches = [];
        this.builder = new PatchBuilder(doc.clock);
    }
    commit() {
        const patch = this.builder.patch;
        this.builder = new PatchBuilder(this.doc.clock);
        if (patch.ops.length) {
            this.patches.push(patch);
            this.doc.applyPatch(patch);
        }
        return this;
    }
    flush() {
        const patches = this.patches;
        this.patches = [];
        return patches;
    }
    flushPatch() {
        const patches = this.flush();
        const length = patches.length;
        const result = new Patch();
        let prev = null;
        for (let i = 0; i < length; i++) {
            const patch = patches[i];
            if (!patch.ops)
                continue;
            if (!prev)
                result.ops.push(...patch.ops);
            else {
                const id = patch.getId();
                const nextTime = prev.nextTime();
                const gap = id.time - nextTime;
                if (gap > 0)
                    result.ops.push(new NoopOperation(new LogicalTimestamp(id.sessionId, nextTime), gap));
                result.ops.push(...patch.ops);
            }
            prev = patch;
        }
        return result;
    }
    patch(cb) {
        this.commit();
        cb(this);
        this.commit();
    }
    root(json) {
        const value = this.builder.json(json);
        this.builder.root(value);
        return this;
    }
    asStr(path) {
        const obj = this.doc.find(path);
        if (obj instanceof StringType)
            return obj;
        throw new Error('NOT_STR');
    }
    str(path) {
        const obj = this.asStr(path);
        return new StringApi(this, obj);
    }
    strObjIns(obj, index, substr) {
        const after = !index ? obj.id : obj.findId(index - 1);
        this.builder.insStr(obj.id, after, substr);
    }
    strIns(path, index, substr) {
        const obj = this.asStr(path);
        this.strObjIns(obj, index, substr);
        return this;
    }
    strObjDel(obj, index, length) {
        const spans = obj.findIdSpan(index, length);
        for (const ts of spans)
            this.builder.del(obj.id, ts, ts.span);
    }
    strDel(path, index, length) {
        const obj = this.asStr(path);
        this.strObjDel(obj, index, length);
        return this;
    }
    asNum(path) {
        const obj = this.doc.find(path);
        if (obj instanceof NumberType)
            return obj;
        throw new Error('NOT_NUM');
    }
    numSet(path, value) {
        const { id } = this.asNum(path);
        this.builder.setNum(id, value);
        return this;
    }
    asArr(path) {
        const obj = this.doc.find(path);
        if (obj instanceof ArrayType)
            return obj;
        throw new Error('NOT_ARR');
    }
    arrIns(path, index, values) {
        const { builder } = this;
        const obj = this.asArr(path);
        const after = !index ? obj.id : obj.findId(index - 1);
        const valueIds = [];
        for (let i = 0; i < values.length; i++)
            valueIds.push(builder.json(values[i]));
        builder.insArr(obj.id, after, valueIds);
        return this;
    }
    arrDel(path, index, length) {
        const obj = this.asArr(path);
        const spans = obj.findIdSpans(index, length);
        for (const ts of spans)
            this.builder.del(obj.id, ts, ts.span);
        return this;
    }
    asObj(path) {
        const obj = this.doc.find(path);
        if (obj instanceof ObjectType)
            return obj;
        throw new Error('NOT_OBJ');
    }
    objSet(path, entries) {
        const obj = this.asObj(path);
        const { builder } = this;
        builder.setKeys(obj.id, Object.entries(entries).map(([key, json]) => [key, builder.json(json)]));
        return this;
    }
    objDel(path, keys) {
        const obj = this.asObj(path);
        const { builder } = this;
        builder.setKeys(obj.id, keys.map(key => [key, UNDEFINED_ID]));
        return this;
    }
}

==> json-joy-2.8.0.tgz: package/esm/json-crdt/api/DocumentApi.js <==
import { StringType } from "../types/rga-string/StringType";
import { PatchBuilder } from "../../json-crdt-patch/PatchBuilder";
import { Patch } from "../../json-crdt-patch/Patch";
import { NoopOperation } from "../../json-crdt-patch/operations/NoopOperation";
import { LogicalTimestamp } from "../../json-crdt-patch/clock";
import { StringApi } from "./StringApi";
import { ArrayType } from "../types/rga-array/ArrayType";
import { ObjectType } from "../types/lww-object/ObjectType";
import { UNDEFINED_ID } from "../../json-crdt-patch/constants";
import { ValueType } from "../types/lww-value/ValueType";
export class DocumentApi {
    constructor(doc) {
        this.doc = doc;
        this.patches = [];
        this.builder = new PatchBuilder(doc.clock);
    }
    commit() {
        const patch = this.builder.patch;
        this.builder = new PatchBuilder(this.doc.clock);
        if (patch.ops.length) {
            this.patches.push(patch);
            this.doc.applyPatch(patch);
        }
        return this;
    }
    flush() {
        const patches = this.patches;
        this.patches = [];
        return patches;
    }
    flushPatch() {
        const patches = this.flush();
        const length = patches.length;
        const result = new Patch();
        let prev = null;
        for (let i = 0; i < length; i++) {
            const patch = patches[i];
            if (!patch.ops)
                continue;
            if (!prev)
                result.ops.push(...patch.ops);
            else {
                const id = patch.getId();
                const nextTime = prev.nextTime();
                const gap = id.time - nextTime;
                if (gap > 0)
                    result.ops.push(new NoopOperation(new LogicalTimestamp(id.sessionId, nextTime), gap));
                result.ops.push(...patch.ops);
            }
            prev = patch;
        }
        return result;
    }
    patch(cb) {
        this.commit();
        cb(this);
        this.commit();
    }
    root(json) {
        const value = this.builder.json(json);
        this.builder.root(value);
        return this;
    }
    asStr(path) {
        const obj = this.doc.find(path);
        if (obj instanceof StringType)
            return obj;
        throw new Error('NOT_STR');
    }
    str(path) {
        const obj = this.asStr(path);
        return new StringApi(this, obj);
    }
    strObjIns(obj, index, substr) {
        const after = !index ? obj.id : obj.findId(index - 1);
        this.builder.insStr(obj.id, after, substr);
    }
    strIns(path, index, substr) {
        const obj = this.asStr(path);
        this.strObjIns(obj, index, substr);
        return this;
    }
    strObjDel(obj, index, length) {
        const spans = obj.findIdSpan(index, length);
        for (const ts of spans)
            this.builder.del(obj.id, ts, ts.span);
    }
    strDel(path, index, length) {
        const obj = this.asStr(path);
        this.strObjDel(obj, index, length);
        return this;
    }
    asVal(path) {
        const obj = this.doc.find(path);
        if (obj instanceof ValueType)
            return obj;
        throw new Error('NOT_VAL');
    }
    valSet(path, value) {
        const { id } = this.asVal(path);
        this.builder.setVal(id, value);
        return this;
    }
    asArr(path) {
        const obj = this.doc.find(path);
        if (obj instanceof ArrayType)
            return obj;
        throw new Error('NOT_ARR');
    }
    arrIns(path, index, values) {
        const { builder } = this;
        const obj = this.asArr(path);
        const after = !index ? obj.id : obj.findId(index - 1);
        const valueIds = [];
        for (let i = 0; i < values.length; i++)
            valueIds.push(builder.json(values[i]));
        builder.insArr(obj.id, after, valueIds);
        return this;
    }
    arrDel(path, index, length) {
        const obj = this.asArr(path);
        const spans = obj.findIdSpans(index, length);
        for (const ts of spans)
            this.builder.del(obj.id, ts, ts.span);
        return this;
    }
    asObj(path) {
        const obj = this.doc.find(path);
        if (obj instanceof ObjectType)
            return obj;
        throw new Error('NOT_OBJ');
    }
    objSet(path, entries) {
        const obj = this.asObj(path);
        const { builder } = this;
        builder.setKeys(obj.id, Object.entries(entries).map(([key, json]) => [key, builder.json(json)]));
        return this;
    }
    objDel(path, keys) {
        const obj = this.asObj(path);
        const { builder } = this;
        builder.setKeys(obj.id, keys.map(key => [key, UNDEFINED_ID]));
        return this;
    }
}
